def to_data(obj: any) -> dict|list:
    if obj == None: 
        return None
    if list == type(obj):
        return [to_data(e) for e in obj]
    f = getattr(obj, 'to_data', None)
    if f is not None:
        return f()
    return obj


def add_if_not_none(obj: any, value: any, key: str=None) -> any:
    if value is None: 
        return obj

    if key is None and list == type(obj):
        obj.append(value)

    if dict == type(obj):
        obj[key] = value

    return obj

def always_a_list(obj: any) -> list: 
    """
    returns a list nomatter what is in the input. 
    if the argument is a list already it gets returned verbatim
    but if it is not, it is wrapped into a list 
    """
    if list == type(obj):
        return obj
    else:
        return [obj]


def list_is_optional(obj: any) -> any:
    """
    returns a dictionary if a dictionary or a list with just one item is provided
    returns None if None is given, or an empty list
    returns a list otherwise
    """
    if list == type(obj):
        if len(obj) == 0:
            return None
        elif len(obj) == 1:
            return obj[0]
    return obj
    
def list_without_nones(lst: list) -> list:
    return [e for e in lst if e is not None]


from setuptools import find_packages, setup

def long_description():
    with open('README.md', 'r') as readme:
        return readme.read()

setup(
    name='diogi',
    packages=find_packages(include=['diogi']),
    version='0.0.2',
    author='Michal Poreba',
    license='MIT',
    description='A library of tiny little functions to make development that little bit easier.',
    long_description=long_description(),
    long_description_content_type='text/markdown',
    url='https://github.com/michalporeba/odis/packages/diogi',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Framework :: Django :: 4.0',
        'Intended Audience :: Developers',
        'Topic :: Software Development'
    ],
    install_requires=[],
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    test_suite='tests'
)


import diogi.functions as f

def always_a_list(func: callable) -> callable: 
    def wrapper(*args, **kwargs):
        return f.always_a_list(func(*args, **kwargs))
    return wrapper 

def list_is_optional(func: callable) -> callable:
    def wrapper(*args, **kwargs):
        return f.list_is_optional(func(*args, **kwargs))
    return wrapper 

def list_without_nones(func: callable) -> callable: 
    def wrapper(*args, **kwargs):
        return f.list_without_nones(func(*args, **kwargs))
    return wrapper